import { z } from 'zod';
import { MAX_HISTORICAL_DATE_MS } from '../../constants';
import { currencyAmountParser, currencyParser } from '../../currencies/parsers';
import { channelIdParser } from '../../channel/parsers';
import { userIdParser } from '../../user/parsers';
import type { PayinRouteResult } from './types';
import { makePositiveIntegerQueryParser } from '../../common/parsers/queryParams';
import { routingStrategyQueryParser } from '../../graph/strategy/parsers';
import { CHARGE_ROUTING_CONTEXT_ID_ROUTE_PARAM_NAME } from './constants';
import { timestampQueryParser } from '../../parsers';

export const routePacketQueryParser = z
  .object({
    userId: userIdParser,
    payeeId: channelIdParser,
    amt: currencyAmountParser,
    unit: currencyParser,
    when: makePositiveIntegerQueryParser('When')
      .refine((n) => n <= MAX_HISTORICAL_DATE_MS, {
        message: `"When" must be less than ${MAX_HISTORICAL_DATE_MS}`,
      })
      .optional(),
  })
  .merge(routingStrategyQueryParser);

export const routePacketResultParser = z.any(/* stub */).transform((v) => v as PayinRouteResult);

export const chargeRoutingContextIdParser = channelIdParser;

export const routeContextParamsParser = z.object({
  [CHARGE_ROUTING_CONTEXT_ID_ROUTE_PARAM_NAME]: chargeRoutingContextIdParser,
});

export const routeContextQueryParser = z.object({
  from: timestampQueryParser.optional(),
});

export const routeContextResultParser = routePacketResultParser;
